'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _asyncGenerator = function () { function AwaitValue(value) { this.value = value; } function AsyncGenerator(gen) { var front, back; function send(key, arg) { return new Promise(function (resolve, reject) { var request = { key: key, arg: arg, resolve: resolve, reject: reject, next: null }; if (back) { back = back.next = request; } else { front = back = request; resume(key, arg); } }); } function resume(key, arg) { try { var result = gen[key](arg); var value = result.value; if (value instanceof AwaitValue) { Promise.resolve(value.value).then(function (arg) { resume("next", arg); }, function (arg) { resume("throw", arg); }); } else { settle(result.done ? "return" : "normal", result.value); } } catch (err) { settle("throw", err); } } function settle(type, value) { switch (type) { case "return": front.resolve({ value: value, done: true }); break; case "throw": front.reject(value); break; default: front.resolve({ value: value, done: false }); break; } front = front.next; if (front) { resume(front.key, front.arg); } else { back = null; } } this._invoke = send; if (typeof gen.return !== "function") { this.return = undefined; } } if (typeof Symbol === "function" && Symbol.asyncIterator) { AsyncGenerator.prototype[Symbol.asyncIterator] = function () { return this; }; } AsyncGenerator.prototype.next = function (arg) { return this._invoke("next", arg); }; AsyncGenerator.prototype.throw = function (arg) { return this._invoke("throw", arg); }; AsyncGenerator.prototype.return = function (arg) { return this._invoke("return", arg); }; return { wrap: function wrap(fn) { return function () { return new AsyncGenerator(fn.apply(this, arguments)); }; }, await: function _await(value) { return new AwaitValue(value); } }; }();

var _crossFetch = require('cross-fetch');

var _crossFetch2 = _interopRequireDefault(_crossFetch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const apiEndPoint = 'https://api.cognitive.microsoft.com/bing/v7.0/images/search';
const defaults = {
  key: null,
  query: null,
  market: null,
  safeSearch: null,
  offset: 0,
  count: 150,
  amount: 2000,
  fetchCb: _crossFetch2.default
};
const apiKeyHeaderName = 'Ocp-Apim-Subscription-Key';
const clientIDHeaderName = 'X-MSEdge-ClientID';
const acceptHeaderName = 'Accept';
const acceptHeaderValue = 'application/json';

function filterNulls(obj) {
  return Object.assign(...Object.keys(obj).filter(key => obj[key] != null).map(key => ({ [key]: obj[key] })));
}

function stringify(obj) {
  return Object.keys(obj).map(k => `${encodeURIComponent(k)}=${encodeURIComponent(obj[k])}`).join('&');
}

exports.default = (() => {
  var _ref = _asyncGenerator.wrap(function* (options) {
    var _Object$assign = Object.assign({}, defaults, options);

    const key = _Object$assign.key,
          query = _Object$assign.query,
          market = _Object$assign.market,
          safeSearch = _Object$assign.safeSearch,
          offset = _Object$assign.offset,
          count = _Object$assign.count,
          amount = _Object$assign.amount,
          queryParams = _Object$assign.queryParams,
          headerParams = _Object$assign.headerParams,
          fetchCb = _Object$assign.fetchCb;

    let currOffset = offset;
    let currAmount = amount;
    let available = currOffset + currAmount;
    let clientID = null;

    while (currOffset < Math.min(currOffset + currAmount, available)) {
      const requestParams = filterNulls(_extends({
        q: query,
        mkt: market,
        safeSearch,
        offset: currOffset,
        count: Math.min(count, Math.min(currOffset + currAmount, available) - currOffset)
      }, queryParams));
      const requestHeaders = filterNulls(_extends({
        [apiKeyHeaderName]: key,
        [clientIDHeaderName]: clientID,
        [acceptHeaderName]: acceptHeaderValue
      }, headerParams));
      const requestOptions = {
        method: 'GET',
        headers: requestHeaders
      };
      const requestUrl = `${apiEndPoint}?${stringify(requestParams)}`;
      const response = yield _asyncGenerator.await(fetchCb(requestUrl, requestOptions));
      const ok = response.ok,
            status = response.status,
            statusText = response.statusText;

      if (!ok) {
        throw new Error(`HTTP error ${status}: "${statusText}"`);
      }
      const body = yield _asyncGenerator.await(response.json());
      currOffset = body.nextOffset;
      currAmount -= body.value.length;
      available = body.totalEstimatedMatches;
      clientID = response.headers.get(clientIDHeaderName);
      yield body;
    }
  });

  function search(_x) {
    return _ref.apply(this, arguments);
  }

  return search;
})();

module.exports = exports['default'];